package kiyut.sketsa.modules.extratool;

import kiyut.sketsa.windows.toolbox.AbstractToolAction;
import org.openide.util.NbBundle;

/** Action wrapper for {@link kiyut.sketsa.modules.extratool.SpiralTool} which is used by Toolbox, aka Toolbox Item
 *
 * @author Kiyut
 */
public class SpiralToolAction extends AbstractToolAction {
    
    /** Creates a new instance of SpiralToolAction */
    public SpiralToolAction() {
        tool = new SpiralTool();
    }
    
    @Override
    public String getName() {
        return NbBundle.getMessage(SpiralToolAction.class, "CTL_SpiralToolAction");
    }
    
    @Override
    protected String iconResource() {
        return NbBundle.getMessage(SpiralToolAction.class, "ICON_SpiralToolAction");
    }
}
